/*!
 * @file main.c
 * @brief Pisca o led em cores diferentes selecionadas de acordo com o intervalo de tempo de aperto 
 *        do push button conectado &agrave; PTA12 e em frequ&ecirc;ncias controladas pelo PTA5. 
 * @mainpage Projeto de pisca-pisca com cores e frequ&ecirc;ncias selecion&aacute;veis
 * @author Wu Shin-Ting
 * @date 20/08/2016
 * @attention Aloca&ccedil;&atilde;o din&acirc;mica de mem&oacute;ria flexibiliza o uso da mem&oacute;ria durante
 * tempo de execu&ccedil;&atilde;o, mas pode degradar o desempenho do sistema. 
 * Deve-se avaliar custoxbenef&iacute;cio. N&atilde;o se esque&ccedil;a de liberar
 * o espa&ccedil;o de mem&oacute;ria alocado dinamicamente ap&oacute;s seu uso.
 * 
 * @note Documenta&ccedil;&atilde;o do c&oacute;digo no formato de Doxygen:
 *       http://mcuoneclipse.com/2014/09/01/automatic-documentation-generation-doxygen-with-processor-expert/
*/
#include <stdlib.h>
#include <derivative.h>

#include "atrasos.h"
#include "ledRGB.h"
#include "pushbutton.h"

/*!
* @brief Executa a fun&ccedil;&atilde;o de inicializa&ccedil;&atilde;o e ap&oacute;s se mant&eacute;m em um la&ccedil;o no qual &eacute; 
*     verificado o bot&atilde;o e em seguida controla a cor dos leds piscantes conforme o intervalo ds tempo que o bot&atilde;o ficou apertado.
* @return 1
*/
int main(void){
    uint8_t i;
    short pta;
    uint8_t indice5 = 5, indice12 = 5;
    float *tmp, *freqs;
    enum _estado_led *cores_dinamica;
    struct _cor cores_estaticas[] = {{ACESO,APAGADO,APAGADO},{ACESO,ACESO,ACESO},
   		{ACESO,ACESO,APAGADO},{APAGADO,ACESO,ACESO},
    		{ACESO,APAGADO,ACESO},{APAGADO,APAGADO,APAGADO}};
    
    /*!
     * Inicializa&ccedil;&otilde;es do GPIO do Kinetis
     */
    initPushbuttonIRQA();
    initLedRGB();

    /*!
     * Aloca&ccedi&atilde;o din&acirc;mica e inicializa&ccedil;&atildeo dos dados
     */
    cores_dinamica = (enum _estado_led *)malloc(18*sizeof(enum _estado_led));
    for(i = 0; i < 6; i++) {
    	cores_dinamica[i*3] = cores_estaticas[i].vermelho;
    	cores_dinamica[i*3+1] = cores_estaticas[i].verde;
    	cores_dinamica[i*3+2] = cores_estaticas[i].azul;
    }

    freqs = (float *)malloc(6*sizeof(float));
    freqs[0] = 0.5;
    freqs[1] = 1.0;
    freqs[2] = 5.0;
    freqs[3] = 15.0;
    freqs[4] = 30.0;
    freqs[5] = 60.0;

    /*! 
     * Execu&ccedil;&atilde;o da tarefa pisca-pisca sob controle
     */
    for(;;){
            pta = le_pta(PTA5);
            /*!
             * Se bot&atilde;o de controle de frequ&ecirc;ncia &eacute; apertado
             */
            if (!pta)
            {
              /*! ent&atilde;o estima o intervalo de tempo que ficou neste estado */
              indice5 = tempo_espera(PTA5);
            }

            pta = le_pta(PTA12);
            /*!
             * Se bot&atilde;o de controle de cor &eacute; apertado
             */
            if (!pta)
            {
              /*! ent&atilde;o estima o intervalo de tempo que ficou neste estado */
              indice12 = tempo_espera(PTA12);
            }
            
            /*!
             * Exibe a cor conforme o intervalo de tempo estimado
             */
        	led (VERMELHO, *((enum _estado_led *)(cores_dinamica+(indice12*3))));
        	led (VERDE, *((enum _estado_led *)(cores_dinamica+(indice12*3+1))));
        	led (AZUL, *((enum _estado_led *)(cores_dinamica+(indice12*3+2))));

        	/*!
        	 * Pisca conforme o intervalo de tempo estimado
        	 */
        	tmp = &freqs[indice5];
        	delayfreq(*tmp);                /*! Espera 1/(2*freqs[indice5]) */
        	       	
            /*!
             * Se o estado dos leds n&atilde;o for apagado (indice12 != 5)
             */
            if (indice12 != 5)
            {
                /*! Alterna o estado dos leds  */
                led (VERMELHO, (cores_estaticas[indice5].vermelho+1)%2);
                led (VERDE, (cores_estaticas[indice5].verde+1)%2);
                led (AZUL, (cores_estaticas[indice5].azul+1)%2);
                
                delayfreq(*tmp);           /*! Espera 1/(2*freqs[indice5]) */
            }

    }

    /*!
     * Libera o espa&ccedil;o da mem&oacute;ria alocada dinamicamente
     */
    free((void *)freqs);
    free((void *)cores_dinamica);

    return 1;
}            
